#!/bin/bash

# simple Disk I/O benchmark test  2018.6.21 by Masahiro Nakajima
# Linux系のシステムを前提

if [ $(whoami) != "root" ]; then # root権限で実行していることを確認
    echo "must execute as root."
    exit
fi

blocks=1000

while getopts b: OPT
do
    case $OPT in
      b) # 読み書きするブロック数を指定
        blocks=$(expr ${OPTARG} + 0) > /dev/null 2>&1
        if [ $? -gt 0 ]; then
            blocks=1000
        elif [ ${blocks} -lt 1 ]; then
            blocks=1000
        fi
        ;;
      ?)
        echo "$0 [-b <#blocks>] target"
        exit 1
        ;;
    esac
done
shift $((OPTIND - 1))

if [ $# -eq 0 ] ; then # デフォルトのベンチマークディレクトリ. 外部メディアは、ここにマウントする
    target=/mnt/target
else
    target=$1
fi

if [ ! -d ${target} ] ; then
    echo "no target directory \"${target}\""
    exit 1
fi

tfile="bm.tmp"

echo "You need at least \" 4 * ${blocks}MB \" free space on \"${target}\"."
read -p "Quit? [y/N]" q
case $q in
    [yY]* ) echo "quit"
            exit 1
    ;;
esac

sync ; echo 3 > /proc/sys/vm/drop_caches  # キャッシュクリア
echo "---- ----"
echo 'starting write test'
time dd if=/dev/zero of=${target}/${tfile} count=${blocks} bs=4M
echo 'write test finished'
sleep 5

sync ; echo 3 > /proc/sys/vm/drop_caches  # キャッシュクリア
echo "---- ----"
echo 'starting read test'
sleep 5
time dd if=${target}/${tfile} of=/dev/null bs=4M
echo 'read test finished'
echo "==== ===="

rm ${target}/${tfile}
